import json
from os import path

from ._replace_bytes_values import _replace_bytes_values
from .s3_get import s3_get


def censor(json_string: str) -> bytes:
    """Replaces values in a file hosted on S3 based on keys present
    in ``private_keys``.
    returning bytes that can be passed to ``put_object``.

    ``json_string`` must contains the keys ``s3_uri`` & ``private_keys``.

    ``s3_uri`` must be a string containing the s3 uri for the file
    to be modified, and the file MUST be a valid csv file.

    ``private_keys`` must be a list containing the keys to be replaced
    """
    params = json.loads(json_string)
    s3_uri: str = params["s3_uri"]
    private_keys: list[str] = params["private_keys"]
    data = s3_get(s3_uri)

    file_format = path.splitext(s3_uri)[1][1:].lower()
    if file_format not in ["csv", "json", "parquet"]:
        raise ValueError(f"Can't handle file format: {file_format}")

    privatized_bytes = _replace_bytes_values(data, private_keys, file_format)
    return privatized_bytes



from droppii._replace_bytes_values import _replace_bytes_values
from droppii.censor import censor
from droppii.replace_df_fields import replace_df_fields
from droppii.s3_get import s3_get

__all__ = [
    "_replace_bytes_values",
    "censor",
    "replace_df_fields",
    "s3_get"
]



import argparse
import json
import sys

from .censor import censor


def main():
    parser = argparse.ArgumentParser(
        prog="droppii",
        description="""Python module to process data from an AWS (S3) bucket
    and anonymize personally
    identifiable information, returning data in the same format as provided."""
    )

    parser.add_argument("-i", "--input", metavar="s3 uri", type=str,
                        required=True, help="s3 uri of file to be converted")
    parser.add_argument("-k", "--keys", type=str,
                        nargs='+', help='Keys to censor')

    args = parser.parse_args()

    json_arguments = json.dumps({
        "s3_uri": args.input,
        "private_keys": args.keys
    })

    censor_output = censor(json_arguments)
    sys.stdout.buffer.write(censor_output)



from io import BytesIO
import json

import polars as pl

from .replace_df_fields import replace_df_fields


def _replace_bytes_values(data: bytes,
                          private_keys: list[str],
                          data_format: str) -> bytes:
    """Replace all values for all keys listed in `private_keys`

    `data` must be a file-like object representing a csv file."""
    buf = BytesIO()
    if data_format == "csv":
        df = pl.read_csv(data)
    elif data_format == "json":
        try:
            df = pl.read_json(data)
        except RuntimeError:
            raise ValueError("Invalid json file")
    elif data_format == "parquet":
        try:
            df = pl.read_parquet(data)
        except pl.exceptions.ComputeError:
            raise ValueError("Invalid Parquet file")
    else:
        raise TypeError("Invalid file provided")

    new_df = replace_df_fields(df, private_keys)
    if new_df.columns != df.columns:
        raise RuntimeError(
            "DataFrame column mismatch in transformed data found")

    if data_format == "csv":
        new_df.write_csv(buf)
    elif data_format == "json":
        new_df.write_json(buf, row_oriented=True)
    elif data_format == "parquet":
        new_df.write_parquet(buf)
    else:
        raise TypeError("Invalid file provided")

    buf.seek(0)
    if data_format in ["csv", "parquet"]:
        privatized_bytes = buf.read()
    else:
        json_data = json.load(buf)
        privatized_bytes = json.dumps(json_data, indent=4).encode()
    return privatized_bytes



import polars as pl


def replace_df_fields(
        df: pl.DataFrame, private_keys: list[str]) -> pl.DataFrame:
    """Return a dataframe with all columns listed in `private_keys`
    replaced with '***'"""
    if not isinstance(df, pl.DataFrame):
        raise TypeError("Not a polars DataFrame")
    for key in private_keys:
        df = df.with_columns(pl.col(key).map_batches(lambda x: "***"))
    return df



import boto3


def s3_get(s3_uri: str) -> bytes:
    """downloads a file from an S3 uri and returns the contents."""
    s3_parts: list[str] = s3_uri.replace("s3://", "").split("/")
    bucket = s3_parts.pop(0)
    file_path = "/".join(s3_parts)
    s3_client = boto3.client(service_name="s3")

    response = s3_client.get_object(Bucket=bucket, Key=file_path)

    data = response["Body"].read()
    return data
